@@ -2,13 +2,13 @@
 
 template <typename T>
 void PermuteResDistinct(T* mat, const std::vector<T> &v,
-                        std::vector<int> &z, int n, int m,
-                        int nRows, const funcPtr<T> myFun) {
+                        std::vector<int> &z, std::size_t n, std::size_t m,
+                        std::size_t nRows, const funcPtr<T> myFun) {
 
     std::vector<T> vPass(m);
-    auto arrPerm = FromCpp14::make_unique<int[]>(n);
+    auto arrPerm = std::make_unique<int[]>(n);
 
-    for (int i = 0; i < n; ++i) {
+    for (std::size_t i = 0; i < n; ++i) {
         arrPerm[i] = z[i];
     }
 
@@ -17,7 +17,7 @@ void PermuteResDistinct(T* mat, const std::vector<T> &v,
         // the result of myFun on v will remain the same for the 5
         // functions defined in ConstraintsUtils.h (i.e. order does
         // not matter for min, max, prod, mean, & sum).
-        for (int j = 0; j < m; ++j) {
+        for (std::size_t j = 0; j < m; ++j) {
             vPass[j] = v[arrPerm[j]];
             mat[nRows * j] = vPass[j];
         }
@@ -26,49 +26,49 @@ void PermuteResDistinct(T* mat, const std::vector<T> &v,
         mat[nRows * m] = myRes;
         nextFullPerm(arrPerm.get(), n - 1);
 
-        for (int count = 1, numR1 = nRows - 1,
+        for (std::size_t count = 1, numR1 = nRows - 1,
              maxInd = n - 1; count < numR1; ++count) {
 
-            for (int j = 0; j < m; ++j) {
-                mat[count +  nRows * j] = v[arrPerm[j]];
+            for (std::size_t j = 0; j < m; ++j) {
+                mat[count + nRows * j] = v[arrPerm[j]];
             }
 
-            mat[count +  nRows * m] = myRes;
+            mat[count + nRows * m] = myRes;
             nextFullPerm(arrPerm.get(), maxInd);
         }
     } else {
-        for (int count = 0, numR1 = nRows - 1, lastCol = m - 1,
+        for (std::size_t count = 0, numR1 = nRows - 1, lastCol = m - 1,
              maxInd = n - 1; count < numR1; ++count) {
 
-            for (int j = 0; j < m; ++j) {
+            for (std::size_t j = 0; j < m; ++j) {
                 vPass[j] = v[arrPerm[j]];
-                mat[count +  nRows * j] = vPass[j];
+                mat[count + nRows * j] = vPass[j];
             }
 
-            mat[count +  nRows * m] = myFun(vPass, m);
+            mat[count + nRows * m] = myFun(vPass, m);
             nextPartialPerm(arrPerm.get(), lastCol, maxInd);
         }
     }
 
     // Get last permutation
-    for (int j = 0; j < m; ++j) {
+    for (std::size_t j = 0; j < m; ++j) {
         vPass[j] = v[arrPerm[j]];
-        mat[nRows - 1L +  nRows * j] = vPass[j];
+        mat[nRows - 1L + nRows * j] = vPass[j];
     }
 
-    mat[nRows - 1L +  nRows * m] = myFun(vPass, m);
+    mat[nRows - 1L + nRows * m] = myFun(vPass, m);
 }
 
 template <typename T>
 void PermuteResDistinct(RcppParallel::RMatrix<T> &mat,
                         const std::vector<T> &v, std::vector<int> &z,
-                        int n, int m, int strt, int nRows,
-                        const funcPtr<T> myFun) {
+                        std::size_t n, std::size_t m, std::size_t strt,
+                        std::size_t nRows, const funcPtr<T> myFun) {
 
     std::vector<T> vPass(m);
-    auto arrPerm = FromCpp14::make_unique<int[]>(n);
+    auto arrPerm = std::make_unique<int[]>(n);
 
-    for (int i = 0; i < n; ++i) {
+    for (std::size_t i = 0; i < n; ++i) {
         arrPerm[i] = z[i];
     }
 
@@ -77,7 +77,7 @@ void PermuteResDistinct(RcppParallel::RMatrix<T> &mat,
         // the result of myFun on v will remain the same for the 5
         // functions defined in ConstraintsUtils.h (i.e. order does
         // not matter for min, max, prod, mean, & sum).
-        for (int j = 0; j < m; ++j) {
+        for (std::size_t j = 0; j < m; ++j) {
             vPass[j] = v[arrPerm[j]];
             mat(strt, j) = vPass[j];
         }
@@ -86,10 +86,10 @@ void PermuteResDistinct(RcppParallel::RMatrix<T> &mat,
         mat(strt, m) = myRes;
         nextFullPerm(arrPerm.get(), n - 1);
 
-        for (int count = strt + 1, numR1 = nRows - 1,
+        for (std::size_t count = strt + 1, numR1 = nRows - 1,
              maxInd = n - 1; count < numR1; ++count) {
 
-            for (int j = 0; j < m; ++j) {
+            for (std::size_t j = 0; j < m; ++j) {
                 mat(count, j) = v[arrPerm[j]];
             }
 
@@ -97,10 +97,10 @@ void PermuteResDistinct(RcppParallel::RMatrix<T> &mat,
             nextFullPerm(arrPerm.get(), maxInd);
         }
     } else {
-        for (int count = strt, numR1 = nRows - 1, lastCol = m - 1,
+        for (std::size_t count = strt, numR1 = nRows - 1, lastCol = m - 1,
              maxInd = n - 1; count < numR1; ++count) {
 
-            for (int j = 0; j < m; ++j) {
+            for (std::size_t j = 0; j < m; ++j) {
                 vPass[j] = v[arrPerm[j]];
                 mat(count, j) = vPass[j];
             }
@@ -111,7 +111,7 @@ void PermuteResDistinct(RcppParallel::RMatrix<T> &mat,
     }
 
     // Get last permutation
-    for (int j = 0; j < m; ++j) {
+    for (std::size_t j = 0; j < m; ++j) {
         vPass[j] = v[arrPerm[j]];
         mat(nRows - 1L, j) = vPass[j];
     }
@@ -121,15 +121,15 @@ void PermuteResDistinct(RcppParallel::RMatrix<T> &mat,
 
 template <typename T>
 void PermuteResRep(T* mat, const std::vector<T> &v,
-                   std::vector<int> &z, int n, int m,
-                   int nRows, const funcPtr<T> myFun) {
+                   std::vector<int> &z, std::size_t n, std::size_t m,
+                   std::size_t nRows, const funcPtr<T> myFun) {
 
     std::vector<T> vPass(m);
 
-    for (int count = 0, maxInd = n - 1,
+    for (std::size_t count = 0, maxInd = n - 1,
          lastCol = m - 1; count < nRows; ++count) {
 
-        for (int j = 0; j < m; ++j) {
+        for (std::size_t j = 0; j < m; ++j) {
             vPass[j] = v[z[j]];
             mat[count + nRows * j] = vPass[j];
         }
@@ -137,7 +137,7 @@ void PermuteResRep(T* mat, const std::vector<T> &v,
         mat[count + nRows * m] = myFun(vPass, m);
 
         for (int i = lastCol; i >= 0; --i) {
-            if (z[i] != maxInd) {
+            if (z[i] != static_cast<int>(maxInd)) {
                 ++z[i];
                 break;
             } else {
@@ -150,15 +150,16 @@ void PermuteResRep(T* mat, const std::vector<T> &v,
 template <typename T>
 void PermuteResRep(RcppParallel::RMatrix<T> &mat,
                    const std::vector<T> &v,
-                   std::vector<int> &z, int n, int m,
-                   int strt, int nRows, const funcPtr<T> myFun) {
+                   std::vector<int> &z, std::size_t n,
+                   std::size_t m, std::size_t strt,
+                   std::size_t nRows, const funcPtr<T> myFun) {
 
     std::vector<T> vPass(m);
 
-    for (int count = strt, maxInd = n - 1,
+    for (std::size_t count = strt, maxInd = n - 1,
          lastCol = m - 1; count < nRows; ++count) {
 
-        for (int j = 0; j < m; ++j) {
+        for (std::size_t j = 0; j < m; ++j) {
             vPass[j] = v[z[j]];
             mat(count, j) = vPass[j];
         }
@@ -166,7 +167,7 @@ void PermuteResRep(RcppParallel::RMatrix<T> &mat,
         mat(count, m) = myFun(vPass, m);
 
         for (int i = lastCol; i >= 0; --i) {
-            if (z[i] != maxInd) {
+            if (z[i] != static_cast<int>(maxInd)) {
                 ++z[i];
                 break;
             } else {
@@ -178,14 +179,15 @@ void PermuteResRep(RcppParallel::RMatrix<T> &mat,
 
 template <typename T>
 void MultisetPermRes(T* mat, const std::vector<T> &v,
-                     std::vector<int> &z, int n, int m, int nRows,
-                     const std::vector<int> &freqs, const funcPtr<T> myFun) {
+                     std::vector<int> &z, std::size_t n, std::size_t m,
+                     std::size_t nRows, const std::vector<int> &freqs,
+                     const funcPtr<T> myFun) {
 
-    const int lenFreqs = freqs.size();
-    auto arrPerm = FromCpp14::make_unique<int[]>(lenFreqs);
+    const std::size_t lenFreqs = freqs.size();
+    auto arrPerm = std::make_unique<int[]>(lenFreqs);
     std::vector<T> vPass(m);
 
-    for (int j = 0; j < lenFreqs; ++j) {
+    for (std::size_t j = 0; j < lenFreqs; ++j) {
         arrPerm[j] = z[j];
     }
 
@@ -194,7 +196,7 @@ void MultisetPermRes(T* mat, const std::vector<T> &v,
         // the result of myFun on v will remain the same for the 5
         // functions defined in ConstraintsUtils.h (i.e. order does
         // not matter for min, max, prod, mean, & sum).
-        for (int j = 0; j < m; ++j) {
+        for (std::size_t j = 0; j < m; ++j) {
             vPass[j] = v[arrPerm[j]];
             mat[j * nRows] = vPass[j];
         }
@@ -203,10 +205,10 @@ void MultisetPermRes(T* mat, const std::vector<T> &v,
         mat[m * nRows] = myRes;
         nextFullPerm(arrPerm.get(), lenFreqs - 1);
 
-        for (int count = 1, numR1 = nRows - 1,
+        for (std::size_t count = 1, numR1 = nRows - 1,
              maxInd = lenFreqs - 1; count < numR1; ++count) {
 
-            for (int j = 0; j < m; ++j) {
+            for (std::size_t j = 0; j < m; ++j) {
                 mat[count + j * nRows] = v[arrPerm[j]];
             }
 
@@ -214,10 +216,10 @@ void MultisetPermRes(T* mat, const std::vector<T> &v,
             nextFullPerm(arrPerm.get(), maxInd);
         }
     } else {
-        for (int count = 0, numR1 = nRows - 1, lastCol = m - 1,
+        for (std::size_t count = 0, numR1 = nRows - 1, lastCol = m - 1,
              maxInd = lenFreqs - 1; count < numR1; ++count) {
 
-            for (int j = 0; j < m; ++j) {
+            for (std::size_t j = 0; j < m; ++j) {
                 vPass[j] = v[arrPerm[j]];
                 mat[count + j * nRows] = vPass[j];
             }
@@ -228,7 +230,7 @@ void MultisetPermRes(T* mat, const std::vector<T> &v,
     }
 
     // Get last permutation
-    for (int j = 0; j < m; ++j) {
+    for (std::size_t j = 0; j < m; ++j) {
         vPass[j] = v[arrPerm[j]];
         mat[nRows - 1 + j * nRows] = vPass[j];
     }
@@ -238,14 +240,15 @@ void MultisetPermRes(T* mat, const std::vector<T> &v,
 
 template <typename T>
 void MultisetPermRes(RcppParallel::RMatrix<T> &mat, const std::vector<T> &v,
-                     std::vector<int> &z, int n, int m, int strt, int nRows,
+                     std::vector<int> &z, std::size_t n, std::size_t m,
+                     std::size_t strt, std::size_t nRows,
                      const std::vector<int> &freqs, const funcPtr<T> myFun) {
 
-    const int lenFreqs = freqs.size();
-    auto arrPerm = FromCpp14::make_unique<int[]>(lenFreqs);
+    const std::size_t lenFreqs = freqs.size();
+    auto arrPerm = std::make_unique<int[]>(lenFreqs);
     std::vector<T> vPass(m);
 
-    for (int j = 0; j < lenFreqs; ++j) {
+    for (std::size_t j = 0; j < lenFreqs; ++j) {
         arrPerm[j] = z[j];
     }
 
@@ -254,7 +257,7 @@ void MultisetPermRes(RcppParallel::RMatrix<T> &mat, const std::vector<T> &v,
         // the result of myFun on v will remain the same for the 5
         // functions defined in ConstraintsUtils.h (i.e. order does
         // not matter for min, max, prod, mean, & sum).
-        for (int j = 0; j < m; ++j) {
+        for (std::size_t j = 0; j < m; ++j) {
             vPass[j] = v[arrPerm[j]];
             mat(strt, j) = vPass[j];
         }
@@ -263,10 +266,10 @@ void MultisetPermRes(RcppParallel::RMatrix<T> &mat, const std::vector<T> &v,
         mat(strt, m) = myRes;
         nextFullPerm(arrPerm.get(), lenFreqs - 1);
 
-        for (int count = strt + 1, numR1 = nRows - 1,
+        for (std::size_t count = strt + 1, numR1 = nRows - 1,
              maxInd = lenFreqs - 1; count < numR1; ++count) {
 
-            for (int j = 0; j < m; ++j) {
+            for (std::size_t j = 0; j < m; ++j) {
                 mat(count, j) = v[arrPerm[j]];
             }
 
@@ -274,10 +277,10 @@ void MultisetPermRes(RcppParallel::RMatrix<T> &mat, const std::vector<T> &v,
             nextFullPerm(arrPerm.get(), maxInd);
         }
     } else {
-        for (int count = strt, numR1 = nRows - 1, lastCol = m - 1,
+        for (std::size_t count = strt, numR1 = nRows - 1, lastCol = m - 1,
              maxInd = lenFreqs - 1; count < numR1; ++count) {
 
-            for (int j = 0; j < m; ++j) {
+            for (std::size_t j = 0; j < m; ++j) {
                 vPass[j] = v[arrPerm[j]];
                 mat(count, j) = vPass[j];
             }
@@ -288,7 +291,7 @@ void MultisetPermRes(RcppParallel::RMatrix<T> &mat, const std::vector<T> &v,
     }
 
     // Get last permutation
-    for (int j = 0; j < m; ++j) {
+    for (std::size_t j = 0; j < m; ++j) {
         vPass[j] = v[arrPerm[j]];
         mat(nRows - 1, j) = vPass[j];
     }
@@ -297,46 +300,52 @@ void MultisetPermRes(RcppParallel::RMatrix<T> &mat, const std::vector<T> &v,
 }
 
 template void PermuteResDistinct(int*, const std::vector<int>&,
-                                 std::vector<int>&, int, int, int,
-                                 const funcPtr<int>);
+                                 std::vector<int>&, std::size_t, std::size_t,
+                                 std::size_t, const funcPtr<int>);
 template void PermuteResDistinct(double*, const std::vector<double>&,
-                                 std::vector<int>&, int, int, int,
-                                 const funcPtr<double>);
+                                 std::vector<int>&, std::size_t, std::size_t,
+                                 std::size_t, const funcPtr<double>);
 template void PermuteResDistinct(RcppParallel::RMatrix<int>&,
                                  const std::vector<int>&, std::vector<int>&,
-                                 int, int, int, int, const funcPtr<int>);
+                                 std::size_t, std::size_t, std::size_t,
+                                 std::size_t, const funcPtr<int>);
 template void PermuteResDistinct(RcppParallel::RMatrix<double>&,
                                  const std::vector<double>&,
-                                 std::vector<int>&, int, int, int,
-                                 int, const funcPtr<double>);
+                                 std::vector<int>&, std::size_t,
+                                 std::size_t, std::size_t,
+                                 std::size_t, const funcPtr<double>);
 
 template void PermuteResRep(int*, const std::vector<int>&, std::vector<int>&,
-                            int, int, int, const funcPtr<int>);
+                            std::size_t, std::size_t, std::size_t,
+                            const funcPtr<int>);
 template void PermuteResRep(double*, const std::vector<double>&,
-                            std::vector<int>&,
-                            int, int, int, const funcPtr<double>);
+                            std::vector<int>&, std::size_t, std::size_t,
+                            std::size_t, const funcPtr<double>);
 template void PermuteResRep(RcppParallel::RMatrix<int>&,
                             const std::vector<int>&,
-                            std::vector<int>&, int, int, int,
-                            int, const funcPtr<int>);
+                            std::vector<int>&, std::size_t, std::size_t,
+                            std::size_t, std::size_t, const funcPtr<int>);
 template void PermuteResRep(RcppParallel::RMatrix<double>&,
                             const std::vector<double>&,
-                            std::vector<int>&, int, int, int,
-                            int, const funcPtr<double>);
+                            std::vector<int>&, std::size_t, std::size_t,
+                            std::size_t, std::size_t, const funcPtr<double>);
 
 template void MultisetPermRes(int*, const std::vector<int>&,
-                              std::vector<int>&, int, int, int,
+                              std::vector<int>&, std::size_t,
+                              std::size_t, std::size_t,
                               const std::vector<int>&, const funcPtr<int>);
 template void MultisetPermRes(double*, const std::vector<double>&,
-                              std::vector<int>&, int, int, int,
+                              std::vector<int>&, std::size_t,
+                              std::size_t, std::size_t,
                               const std::vector<int>&,
                               const funcPtr<double>);
 template void MultisetPermRes(RcppParallel::RMatrix<int>&,
                               const std::vector<int>&,
-                              std::vector<int>&, int, int, int, int,
+                              std::vector<int>&, std::size_t,
+                              std::size_t, std::size_t, std::size_t,
                               const std::vector<int>&, const funcPtr<int>);
 template void MultisetPermRes(RcppParallel::RMatrix<double>&,
                               const std::vector<double>&,
-                              std::vector<int>&, int, int, int, int,
-                              const std::vector<int>&,
-                              const funcPtr<double>);
+                              std::vector<int>&, std::size_t,
+                              std::size_t, std::size_t, std::size_t,
+                              const std::vector<int>&, const funcPtr<double>);
